#include <stdio.h>
#include "system.h"
#include "string.h"
#include "altera_avalon_pio_regs.h"
#include <unistd.h>
#include "io.h"

#define NUMBER_OF_WORDS_PER_MS 1250000 /* 1250 Mbps */
#define NUMBER_OF_MS_PER_HOUR 3600000
#define NUMBER_OF_MS_PER_MINUTE 60000
#define NUMBER_OF_MS_PER_SECOND 1000
#define NUMBER_OF_LINKS 4
#define PRBS_7 0 
#define PRBS_23 1
#define PRBS_31 2
#define COUNT_PATTERN 3

/*-----------------------------------------------
 * input_char()
 * 
 * Gets a single input char for menu prompt
 * 
 *   ******** Added by Guy for Menu inputs
 *   ******** Changed by Maurizio for Nios II SDK Shell support
 * 
 * ----------------------------------------------*/
char input_char(void)
 {
 char in_char, rx_char;
 
   rx_char = 0;
   while (1)
   {
     in_char = getchar();
     if (in_char != '\r' && in_char != '\n')
         rx_char = in_char;
     else if (in_char == '\n')
         break;
   };
 return (rx_char);
 }
 
int main()
{ 
	char rx_char;
  int Bitrate;
  int Control_Reg;

  int Channel_Reg[NUMBER_OF_LINKS]; 
  int Counter_1ms_Reg;
  float BER[NUMBER_OF_LINKS];

    
  float Totalbits;
  int Hours;
  int Minutes;
  int Seconds;
  int temp;
  
  int CountPattern;
  
  int Locked_Ch[NUMBER_OF_LINKS];

  
  int tx_locked;
  int rx_locked;

  int ErrorCount_Ch[NUMBER_OF_LINKS];
 
  
  int SelectedChannel;

  int i;


  //char line[100]; 
  int PrbsSelect[NUMBER_OF_LINKS];
  int PrbsPattern[NUMBER_OF_LINKS];

    
  
 Bitrate =  NUMBER_OF_WORDS_PER_MS/1000;
    
  printf("Test for the Stratix IV GX Kit with 4 Channels @ %4d Mbps \n",Bitrate);
  printf("============================================================ \n");
  printf("\n");
  
  
   /**********************************************************************/
  /*  Set PRBS-Pattern to PRBS23                                        */
  /**********************************************************************/  
  
    Control_Reg = 0x0000; 
    
  for (i = 0; i < NUMBER_OF_LINKS ; i++)
  {
	  PrbsSelect[i] = 0x1; 
  }


     Control_Reg = (Control_Reg & (0xFCFF)) | (PrbsSelect[0] << 8); 
     IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
     
     for (i = 0; i < NUMBER_OF_LINKS ; i++)
     {
     Control_Reg = (Control_Reg & (0xFFFC)) | ( i ); // Select channel
     Control_Reg = Control_Reg | (0x1000); /* Latch PrbsPattern Assert */
     Control_Reg = (Control_Reg & (0xFCFF)) | (PrbsSelect[i] << 8); 
     IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
     
     Control_Reg = Control_Reg & (0xEFFF); /* Latch PrbsPattern Deassert */
     IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
     }
     
     
  Control_Reg = 0x0000; 
  IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
  
  CountPattern = 0; 
  SelectedChannel = 0;
  
  Control_Reg = Control_Reg | (0x8000);
IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
usleep(10);

Control_Reg = Control_Reg & (0x7FFF);

IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
usleep(100000);
        
  
  while (1)
	{
  
  Channel_Reg[0]      = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL0_REG_BASE);
  Channel_Reg[1]      = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL1_REG_BASE);
  Channel_Reg[2]      = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL2_REG_BASE);
  Channel_Reg[3]      = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL3_REG_BASE);


  Counter_1ms_Reg   = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE);


  tx_locked   = 0x0001 & (Channel_Reg[0] >> 15);  
  rx_locked   = 0x0001 & (Channel_Reg[0] >> 13);

  
  for (i = 0; i < NUMBER_OF_LINKS ; i++)
  {
  Locked_Ch[i]  = 0x0001 & (Channel_Reg[i] >> 14);
  if (Locked_Ch[i] == 1)
    ErrorCount_Ch[i] = 0x007F & (Channel_Reg[i]);
  else
    ErrorCount_Ch[i] = 99;
  }
                                                           
     
  Totalbits = (float) (Counter_1ms_Reg) * (float) (NUMBER_OF_WORDS_PER_MS);
  
  Hours =  Counter_1ms_Reg / NUMBER_OF_MS_PER_HOUR ;
  Minutes = (Counter_1ms_Reg - (Hours * NUMBER_OF_MS_PER_HOUR))/NUMBER_OF_MS_PER_MINUTE;
  Seconds = (Counter_1ms_Reg - (Hours * NUMBER_OF_MS_PER_HOUR) - (Minutes * NUMBER_OF_MS_PER_MINUTE)) / NUMBER_OF_MS_PER_SECOND;

  for (i = 0; i < NUMBER_OF_LINKS ; i++)
  {

  if (Counter_1ms_Reg > 0)
     {
      if (ErrorCount_Ch[i] > 0)
        BER[i] = ((float)(ErrorCount_Ch[i])) / Totalbits;
      else
        BER[i] = ((float) 3) / Totalbits; /* confidence level (CL) of 95% = -ln(1-CL) = 3*/
     }
  else
    {
    BER[i] = 1; 
    }
  } /* For Loop */ 
  

  for (i = 0; i < NUMBER_OF_LINKS ; i++)
  {
    switch(PrbsSelect[i])
    {
      case 0 : PrbsPattern[i] = 7;break;
      case 1 : PrbsPattern[i] = 23;break;
      case 2 : PrbsPattern[i] = 31;break;
      case 3 : PrbsPattern[i] = 0;break;
      default:break;
    }
  } /* For Loop */   


  if (rx_locked == 0)
    {
    printf("RxPLL is not Stable, no Lock achieved\n");
    }
  else
  {
    printf("\n\n\n");
    printf("Status TestDesign %1d Ch @ %3d Mbps\n",NUMBER_OF_LINKS,Bitrate);
    printf("====================================\n");
    if (PrbsSelect[0] == COUNT_PATTERN)
		printf("|Channel 0:HSMA[7]  (SFP4)  Counter Bitrate %4d Mbps |\n",Bitrate);
	else
		printf("|Channel 0:HSMA[7]  (SFP4)  PRBS-%2d Bitrate %4d Mbps |\n",PrbsPattern[0],Bitrate);
    if (PrbsSelect[1] == COUNT_PATTERN)
		printf("|Channel 1:HSMA[10] (SFP5)  Counter Bitrate %4d Mbps |\n",Bitrate);
	else
		printf("|Channel 1:HSMA[10] (SFP5)  PRBS-%2d Bitrate %4d Mbps |\n",PrbsPattern[1],Bitrate);
    if (PrbsSelect[2] == COUNT_PATTERN)
		printf("|Channel 2:HSMA[12] (SFP6)  Counter Bitrate %4d Mbps |\n",Bitrate);
	else
		printf("|Channel 2:HSMA[12] (SFP6)  PRBS-%2d Bitrate %4d Mbps |\n",PrbsPattern[2],Bitrate);
    if (PrbsSelect[3] == COUNT_PATTERN)
		printf("|Channel 3:HSMA[15] (SFP7)  Counter Bitrate %4d Mbps |\n",Bitrate);
	else
		printf("|Channel 3:HSMA[15] (SFP7)  PRBS-%2d Bitrate %4d Mbps |\n",PrbsPattern[3],Bitrate);						

        
    printf("TxPLL_Locked    : %d\n",tx_locked);
    printf("RxPLL_Locked    : %d\n",rx_locked);    
    
    printf("SelectedChannel : %d\n",SelectedChannel);
    printf("\n");
    
    printf("Channel         :|");
    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    { 
    printf("%2X|",i);
    }
    printf("\n");
    printf("                 |");
    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    { 
    printf("--|");
    }
    printf("\n");
        
    printf("Locked          :|");
    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    { 
    printf("%2d|",Locked_Ch[i]);
    }
    printf("\n");
    
    printf("Errorcount      :|");
    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    { 
    if (Locked_Ch[i] == 1) 
		{
		if (ErrorCount_Ch[i] >= 99)
			printf("XX|");
		else
			printf("%2d|",ErrorCount_Ch[i]);
		}
	else
		printf("  |");
    }
    printf("\n");
    printf("\n");
    
    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    {    
      if (Locked_Ch[i] == 1)
      {
        if (ErrorCount_Ch[i] > 0)
              printf("BER Estimate  Ch %1X  : %e    (# Errors/Totalbits)\n",i,BER[i]);
          else
              printf("BER (CL=0.95) Ch %1X  : %e\n",i,BER[i]);
      }
      else 
              printf("No Lock on Ch %1X\n",i);
    } 

                                                                      
    printf("\nTest Time     : %2dh %2dm %2ds\n\n",Hours,Minutes,Seconds);
  }
  
    printf("\nSelect Action : \n");
    printf("===============\n");
    
    printf("1. Start Test or Do Full Reset\n");
    printf("2. Select Channel To Control\n");
    printf("3. Insert Biterror on Selected Channel\n");
    printf("4. Reset ErrorCounter on Selected Channel\n");
    printf("5. Change Datapattern on Selected Channel\n");
    printf("6. Show Status\n");
    printf("0. Stop Test \n\n");
    printf("  Enter Choice :");
  
      rx_char = input_char(); 
	
		switch (rx_char)
		{
      case '1': /*Start Test */
        Control_Reg = Control_Reg | (0x8000);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
        usleep(10);
        
        Control_Reg = Control_Reg & (0x7FFF);
        
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
        printf("Test has started.\n\n\n");
        usleep(10000);
        break;
        
			case '2': /*Select Channel To Control*/

      printf("2. Select Channel To Control (Hexadecimal)\n");
      printf("    (Note : 'T' will select all channels)\n");
      printf("    Current Channel Selected   :%c\n", SelectedChannel);

    temp = 0xffffffff;
  
     do {
      printf("    New Channel (0-3,T)        :");
      rx_char = input_char(); 
  
      switch (rx_char)
      {
      case '0': temp = 0; break;
      case '1': temp = 1; break;
      case '2': temp = 2; break;
      case '3': temp = 3; break;
      case 'T':
      case 't': temp = 7; break;
      default : break;
      }
 
    } while (temp == 0xffffffff);
    Control_Reg = (Control_Reg & (0xFFF8)) | (temp);     
    
    SelectedChannel = temp; 
                   
    IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);

				
				break;
        
			case '3': /* Insert Biterrors */
     
        printf("3. Insert Biterror On Selected Channel(s)\n\n\n");

  
        Control_Reg = Control_Reg | (0x4000);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
        
        Control_Reg = Control_Reg & (0xBFFF);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);            
            
				break;
	
			case '4': /* Reset ErrorCounter */
        
	      printf("4. Reset ErrorCounter on Selected Channel(s)\n\n\n");
    
        Control_Reg = Control_Reg | (0x2000);    
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);

        Control_Reg = Control_Reg & (0xDFFF);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
        usleep(1000);       
            
       break;
       
  


      case '5': /*Select Prbs Pattern*/

      printf("5. Select Prbs Pattern on Selected Channel and Reset\n");
      printf("    Current Prbs Pattern Selected   : ");
      switch (PrbsSelect[SelectedChannel])
       {
       case  0  : printf(" PRBS-7\n"); break;
       case  1  : printf(" PRBS-23\n"); break;
       case  2  : printf(" PRBS-31\n"); break;
       case  3  : printf(" CountPattern\n"); break;
       default : break;
       }  
     
     
        do {
        printf("\n");  
        
        printf("    New Prbs Pattern           \n");
        printf("    ============================                \n");
        printf("    For PRBS-7          choose '0' \n");
        printf("    For PRBS-23         choose '1' \n");
        printf("    For PRBS-31         choose '2' \n");        
        printf("    For Countpattern    choose '3' \n");                              
        printf("    No Change           choose 'N' \n\n");    
        printf("    Choice :");             
        
        temp = 0xffffffff;
                                             
        rx_char   = input_char();
        switch (rx_char)
        {
        case '0': temp =  PRBS_7; break;
        case '1': temp =  PRBS_23; break;
        case '2': temp =  PRBS_31; break;   
        case '3': temp =  COUNT_PATTERN;break ;                           
        case 'n': 
        case 'N': temp =  PrbsSelect[SelectedChannel];break;
        default : break;
        }
        } while (temp == 0xffffffff);
 
        PrbsSelect[SelectedChannel] = temp; 
        

        Control_Reg = (Control_Reg & (0xFCFF)) | (PrbsSelect[SelectedChannel] << 8); 
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
        

        Control_Reg = Control_Reg | (0x1000);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg); // Latch_PrbsPattern asserted
        
        Control_Reg = Control_Reg & (0xEFFF);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg); // Latch_PrbsPattern de-asserted        
          
        Control_Reg = Control_Reg | (0x8000);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
        usleep(10);
        
        Control_Reg = Control_Reg & (0x7FFF);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
        

      usleep(100000); 
      break; //case '5'

       	
      case '6': /* Show Status */ 
            
       break;    
       
               

                   
     case '0':
        printf("\nTerminating Program.\n");
        return 0;
       break;
  
      default:
        printf("\n\tPlease enter an integer value from 0 to 6\n\n\n");
        break;
    }
 }
 return 0;
}
